@@ -150,8 +150,9 @@ info_load_iface (Netinfo *info)
 	GtkCellRenderer *renderer;
 	GList *items = NULL;
 	GList *p;
-	GdkPixbuf *pixbuf;
-	gchar *iface, *text;
+	GdkPixbuf *pixbuf = NULL;
+	gchar *iface = NULL;
+	gchar *text;
 
 	items = info_get_interfaces (info);
 	p = items;
@@ -336,6 +337,23 @@ info_ip6_masklen (struct in6_addr netmask)
 	return len;
 }
 
+typedef struct {
+	   gchar *ip_addr;
+	   gchar *ip_prefix;
+	   gchar *ip_bcast;
+	   gchar *ip_scope;
+} InfoIpAddr;
+
+static void
+info_ip_addr_free (InfoIpAddr *ip)
+{
+	g_free (ip->ip_addr);
+	g_free (ip->ip_prefix);
+	g_free (ip->ip_bcast);
+	g_free (ip->ip_scope);
+	g_free (ip);
+}
+
 static InfoIpAddr *
 info_ip6_construct_address (const struct ifaddrs *ifr6)
 {
@@ -433,7 +451,7 @@ info_get_nic_information (const gchar *nic, Netinfo *info)
 					    3, ip->ip_bcast,
 					    4, ip->ip_scope,
 					    -1);
-			g_free (ip);
+			info_ip_addr_free (ip);
 
 			break;
 		case AF_INET:
@@ -576,7 +594,8 @@ info_get_nic_information (const gchar *nic, Netinfo *info)
 					    3, ip->ip_bcast,
 					    4, "",
 					    -1);
-			g_free (ip);
+
+			info_ip_addr_free (ip);
 
 			break;
 		default:
@@ -588,12 +607,7 @@ info_get_nic_information (const gchar *nic, Netinfo *info)
 			gtk_label_set_text (GTK_LABEL (info->mtu), NOT_AVAILABLE);
 			gtk_label_set_text (GTK_LABEL (info->state), NOT_AVAILABLE);
 			gtk_label_set_text (GTK_LABEL (info->multicast), NOT_AVAILABLE);
-			
-			if (data.has_data) {
-				gtk_label_set_text (GTK_LABEL (info->link_speed), data.media);
-			} else {
-				gtk_label_set_text (GTK_LABEL (info->link_speed), NOT_AVAILABLE);
-			}
+			gtk_label_set_text (GTK_LABEL (info->link_speed), NOT_AVAILABLE);
 			
 			break;
 		}
