@@ -18,13 +18,17 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
+#include <unistd.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <signal.h>
 #include <errno.h>
 #include <sys/wait.h>
 #include <netdb.h>
+#include <fcntl.h>
 
 #include "nettool.h"
 #include "utils.h"
@@ -67,13 +71,9 @@ void
 netinfo_set_host (Netinfo * netinfo, const gchar *host)
 {
 	g_return_if_fail (netinfo != NULL);
-	g_return_if_fail (GTK_IS_ENTRY
-			  (gtk_bin_get_child
-			   (GTK_BIN (netinfo->host))));
-	
-	gtk_entry_set_text (GTK_ENTRY
-			    (gtk_bin_get_child
-			     (GTK_BIN (netinfo->host))), host);
+	g_return_if_fail (GN_IS_COMBO_HISTORY (netinfo->history));
+
+	gn_combo_history_add (GN_COMBO_HISTORY (netinfo->history), host);
 }
 
 gboolean
@@ -115,13 +115,9 @@ void
 netinfo_set_user (Netinfo * netinfo, const gchar *user)
 {
 	g_return_if_fail (netinfo != NULL);
-	g_return_if_fail (GTK_IS_ENTRY
-			  (gnome_entry_gtk_entry
-			   (GNOME_ENTRY (netinfo->user))));
-	
-	gtk_entry_set_text (GTK_ENTRY
-			    (gnome_entry_gtk_entry
-			     (GNOME_ENTRY (netinfo->user))), user);
+	g_return_if_fail (GN_IS_COMBO_HISTORY (netinfo->history_user));
+
+	gn_combo_history_add (GN_COMBO_HISTORY (netinfo->history_user), user);
 }
 
 gint
@@ -171,22 +167,18 @@ netinfo_error_message (Netinfo     * netinfo,
 		       const gchar * secondary)
 {
 	GtkWidget *dialog;
-	gchar     *message;
  
 	g_return_if_fail (primary != NULL);
 
-	if (secondary)
-		message = g_strdup_printf ("<b><big>%s</big></b>\n\n%s",
-					   primary, secondary);
-	else
-		message = g_strdup_printf ("<b><big>%s</big></b>",
-					   primary);
+	dialog = gtk_message_dialog_new (GTK_WINDOW (netinfo->main_window),
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE,
+					 primary);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  secondary ? secondary : " ");
+	gtk_window_set_title (GTK_WINDOW (dialog), "");
 
-	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (netinfo->main_window),
-						     GTK_DIALOG_DESTROY_WITH_PARENT,
-						     GTK_MESSAGE_ERROR,
-						     GTK_BUTTONS_CLOSE,
-						     message);
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
@@ -207,9 +199,9 @@ netinfo_validate_host (Netinfo * netinfo)
 		secondary = g_strdup (_("Please enter a valid network address and try again."));
 	}
 	else {
-		hostname = gethostbyname2 (host, PF_INET6);
+		hostname = gethostbyname2 (host, PF_INET);
 		if (hostname == NULL) {
-			hostname = gethostbyname2 (host, AF_INET);
+			hostname = gethostbyname2 (host, PF_INET6);
 			if (hostname == NULL) {
 				primary = g_strdup_printf (_("The address '%s' cannot be found"),
 							   host);
@@ -285,6 +277,10 @@ netinfo_text_buffer_insert (Netinfo * netinfo)
 
 		netinfo->child_pid = child_pid;
 		netinfo->pipe_out = pout;
+		fcntl (pout, F_SETFL, O_NONBLOCK);
+		fcntl (pout, F_SETFL, O_NONBLOCK);
+		netinfo->command_output = NULL;
+
 		/*netinfo->pipe_err = perr; */
 
 		channel = g_io_channel_unix_new (pout);
@@ -318,8 +314,8 @@ gboolean
 netinfo_io_text_buffer_dialog (GIOChannel * channel,
 			       GIOCondition condition, gpointer data)
 {
-	gchar *text;
-	gint len;
+	gchar *text = NULL;
+	gsize len;
 	Netinfo *netinfo = (Netinfo *) data;
 
 	g_return_val_if_fail (channel != NULL, FALSE);
@@ -347,18 +343,40 @@ netinfo_io_text_buffer_dialog (GIOChannel * channel,
 #endif				/* DEBUG */
 
 	if (condition & G_IO_IN) {
-		g_io_channel_read_line (channel, &text, &len, NULL, NULL);
+		GIOStatus status;
+
+		status = g_io_channel_read_line (channel, &text, &len, NULL, NULL);
+
+		if (status == G_IO_STATUS_NORMAL) {
+			if (netinfo->command_output) {
+				g_string_append (netinfo->command_output, text);
+				g_free (text);
+				text = g_string_free (netinfo->command_output, FALSE);
+				netinfo->command_output = NULL;
+			}
 
-		if (text != NULL) {
 			if (netinfo->process_line != NULL) {
 				(netinfo->process_line) ((gpointer) netinfo, text,
 						 	len, NULL);
 			}
 
-			g_free (text);
+		} else if (status == G_IO_STATUS_AGAIN) {
+			char buf[1];
 
-			return TRUE;
+			/* A non-terminated line was read, read the data into the buffer. */
+			status = g_io_channel_read_chars (channel, buf, 1, NULL, NULL);
+			if (status == G_IO_STATUS_NORMAL) {
+				if (netinfo->command_output == NULL) {
+					netinfo->command_output = g_string_new (NULL);
+				}
+				g_string_append_c (netinfo->command_output, buf[0]);
+			}
+		} else if (status == G_IO_STATUS_EOF) {
 		}
+
+		g_free (text);
+
+		return TRUE;
 	}
 
 	/* The condition is not G_IO_HUP | G_IO_ERR | G_IO_NVAL, so
